@@ -5,7 +5,7 @@ use Mojolicious::Plugin::Vparam::Common qw(:all);
 use version;
 use List::MoreUtils qw(firstval natatime mesh);
 
-our $VERSION    = '3.00';
+our $VERSION    = '3.03';
 
 # Regext for shortcut parser
 our $SHORTCUT_REGEXP = qr{
@@ -270,11 +270,33 @@ sub register {
 
             # Get value
             my @input;
-            if( $attr{jpath} ) {
+            if ($attr{'jpath?'}) {
                 # JSON Pointer
-                $vars->{json} //= Mojolicious::Plugin::Vparam::JSON::parse_json(
-                    $self->req->body // ''
-                );
+                unless (exists $vars->{json}) {
+                    $vars->{json} =
+                        Mojolicious::Plugin::Vparam::JSON::parse_json(
+                            $self->req->body // ''
+                        );
+                }
+                if( $vars->{json} ) {
+                    $vars->{pointer} //=
+                        Mojo::JSON::Pointer->new( $vars->{json} );
+                    if( $vars->{pointer}->contains( $attr{'jpath?'} ) ) {
+                        my $value = $vars->{pointer}->get( $attr{'jpath?'} );
+                        @input = 'ARRAY' eq ref $value ? @$value : $value;
+                    }
+                } else {
+                    # POST parameters
+                    @input = params($self, $name);
+                }
+            } elsif ($attr{jpath}) {
+                # JSON Pointer
+                unless (exists $vars->{json}) {
+                    $vars->{json} =
+                        Mojolicious::Plugin::Vparam::JSON::parse_json(
+                            $self->req->body // ''
+                        );
+                }
                 if( $vars->{json} ) {
                     $vars->{pointer} //=
                         Mojo::JSON::Pointer->new( $vars->{json} );
@@ -283,22 +305,27 @@ sub register {
                         @input = 'ARRAY' eq ref $value ? @$value : $value;
                     }
                 }
-            } elsif( $attr{cpath} ) {
+            } elsif ($attr{cpath}) {
                 # CSS
-                $vars->{dom} //= Mojolicious::Plugin::Vparam::DOM::parse_dom(
-                    $self->req->body // ''
-                );
+                unless (exists $vars->{dom}) {
+                    $vars->{dom} =
+                        Mojolicious::Plugin::Vparam::DOM::parse_dom(
+                            $self->req->body // ''
+                        );
+                }
                 if( $vars->{dom} ) {
-                    @input = $vars->{dom}->find( $attr{cpath} )
-                        ->map('text')->each;
+                    @input =
+                        $vars->{dom}->find($attr{cpath})->map('text')->each;
+                }
+            } elsif ($attr{xpath}) {
+                unless (exists $vars->{xml}) {
+                    $vars->{xml} = Mojolicious::Plugin::Vparam::XML::parse_xml(
+                        $self->req->body // ''
+                    );
                 }
-            } elsif( $attr{xpath} ) {
-                $vars->{xml} //= Mojolicious::Plugin::Vparam::XML::parse_xml(
-                    $self->req->body // ''
-                );
                 if( $vars->{xml} ) {
                     @input = map {$_->textContent}
-                        $vars->{xml}->findnodes( $attr{xpath} );
+                        $vars->{xml}->findnodes($attr{xpath});
                 }
             } else {
                 # POST parameters
@@ -344,10 +371,8 @@ sub register {
             my @output;
             for my $index ( 0 .. $#input ) {
                 my $in = my $out = $input[$index];
-
-                $out = $in;
-
                 my $key;
+
                 if( $attr{hash} ) {
                     my @splitted = split $attr{hash}, $out, 2;
                     unless( @splitted == 2 ) {
@@ -363,7 +388,7 @@ sub register {
                     }
 
                     $key = $splitted[0];
-                    $out = $splitted[1];
+                    $in = $out = $splitted[1];
                 }
 
                 # Apply pre filter
@@ -377,6 +402,7 @@ sub register {
 
                         # Default value always supress error
                         $error = 0 if exists $attr{default};
+
                         # Disable error on optional
                         if( $attr{optional} ) {
                             # Only if input param not set
@@ -1038,10 +1064,10 @@ Russian date format like C<DD.MM.YYYY>
 
 Boolean value. Can be used to get value from checkbox or another sources.
 
-HTML forms do not send checbox if it checked off.
-You need always set default value to supress error if checkbox not checked:
+HTML forms do not send checbox if it checked off. So you don`t get error,
+but get false for it.
 
-    $self->vparam(mybox => 'bool', default => 0);
+    $self->vparam(mybox => 'bool');
 
 Valid values are:
 
@@ -1375,7 +1401,7 @@ Vparam always return scalars if disabled.
 Note: if defined I<date>, I<time>, I<datetime> then always return
 formatted scalar.
 
-=head2 jpath
+=head2 jpath or jpath?
 
 If you POST data not form but raw JSON you can use JSON Pointer selectors
 from L<Mojo::JSON::Pointer> to get and validate parameters.
@@ -1391,6 +1417,30 @@ from L<Mojo::JSON::Pointer> to get and validate parameters.
 
 Note: we don`t support multikey in json. Use hash or die.
 
+If You use C<jpath?> instead C<jpath>, vparam tries parse input json, if
+json is invalid vparam tries fetch param from input form:
+
+
+    ######################## works:
+    # {"point":{"address":"some", "lon": 45.123456, "lat": 38.23452}}
+
+    # #######################works:
+    # address=some&lon=45.123456&lat=38.23452
+
+    ################# doesn't work:
+    # query: address=some
+    # body:  {"point":{"lon": 45.123456, "lat": 38.23452}}
+
+    %opts = $self->vparams(
+        address => { type => 'str', 'jpath?' => '/point/address' },
+        lon     => { type => 'lon', 'jpath?' => '/point/lon' },
+        lat     => { type => 'lat', 'jpath?' => '/point/lat' },
+    );
+
+Note: You cant mix C<jpath> and C<jpath?>: If body contains valid JSON, vparam
+doesn't try check form params.
+
+
 =head2 cpath
 
 Same as jpath but parse XML/HTML using CSS selectors from L<Mojo::DOM::CSS>.
